
from django.conf import settings
from django.core.exceptions import AppRegistryNotReady


DEFAULT_SHEPHERD_JS = 'https://cdn.jsdelivr.net/npm/shepherd.js@latest/dist/js/shepherd.min.js'
DEFAULT_SHEPHERD_CSS = 'https://cdn.jsdelivr.net/npm/shepherd.js@latest/dist/css/shepherd.css'

TOURS_SHEPHERD_JS = getattr(settings, 'TOURS_SHEPHERD_JS', DEFAULT_SHEPHERD_JS)
TOURS_SHEPHERD_CSS = getattr(settings, 'TOURS_SHEPHERD_CSS', DEFAULT_SHEPHERD_CSS)




from django.contrib import admin

from .forms import TourForm, TourStepForm
from .models import Tour, TourStep


class TourAdmin(admin.ModelAdmin):
    form = TourForm
    list_display = (
        'name',
        'show_only_staff',
        'show_only_superuser',
        'start_date',
        'end_date',
        'is_active',
    )
    list_filter = (
        'show_only_staff',
        'show_only_superuser',
        'start_date',
        'end_date',
        'is_active',
    )
    readonly_fields = (
        'users_shown',
    )
    search_fields = (
        'name__icontains',
        'description__icontains',
    )


class TourStepAdmin(admin.ModelAdmin):
    form = TourStepForm
    list_display = (
        'step_id',
        'tour',
        'title',
    )
    list_filter = (
        'tour',
    )
    search_fields = (
        'title__icontains',
        'text__icontains',
        'step_id__iexact',
        'attach_to_selector__iexact',
    )


admin.site.register(Tour, TourAdmin)
admin.site.register(TourStep, TourStepAdmin)


from django import template
from django.conf import settings
from django.http.request import HttpRequest
from django.db.models import Q

from tours.models import Tour, TourStep
from tours.settings import TOURS_SHEPHERD_CSS, TOURS_SHEPHERD_JS


register = template.Library()


@register.inclusion_tag('tours/tags/tours.html')
def load_tours(request: HttpRequest):
    resolver_match = getattr(request, 'resolver_match', None)

    if resolver_match is not None:
        url_name = resolver_match.url_name

        # Filter active tours related to the current view
        tours = Tour.objects.filter(
            Q(url_names='') |
            Q(url_names__contains=url_name)
        ).exclude(
            is_active=False,
        )

        if getattr(request.user, 'is_authenticated', False):
            tours = tours.exclude(users_shown=request.user)

        if not getattr(request.user, 'is_superuser', False):
            tours = tours.exclude(show_only_superuser=True)

        if not getattr(request.user, 'is_staff', False):
            tours = tours.exclude(show_only_staff=True)

        return {
            'request': request,
            'tours': tours,
            'shepherd_css': TOURS_SHEPHERD_CSS,
            'shepherd_js': TOURS_SHEPHERD_JS,
            'DEBUG': settings.DEBUG,
        }



from django.shortcuts import get_object_or_404, render
from django.http.request import HttpRequest
from django.http.response import JsonResponse
from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_POST

from .models import Tour


@login_required
@require_POST
def complete_tour_view(request: HttpRequest, pk):
    tour = get_object_or_404(Tour, pk=pk)
    tour.users_shown.add(request.user)
    tour.save()
    return JsonResponse({'ok': True})



# Generated by Django 4.2.7 on 2023-12-04 07:25

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Tour',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='name')),
                ('description', models.CharField(blank=True, max_length=200)),
                ('url_names', models.CharField(blank=True, help_text='Comma separated list of url_name where you want it to be displayed. If not specified, it will be displayed on all pages.', max_length=500, verbose_name='url names')),
                ('show_only_staff', models.BooleanField(default=False, verbose_name='show only staff users')),
                ('show_only_superuser', models.BooleanField(default=False, verbose_name='show only staff users')),
                ('start_date', models.DateField(blank=True, null=True, verbose_name='start date')),
                ('end_date', models.DateField(blank=True, null=True, verbose_name='end date')),
                ('timeout', models.PositiveIntegerField(default=1000)),
                ('use_modal_overlay', models.BooleanField(default=True)),
                ('is_active', models.BooleanField(default=True)),
                ('users_shown', models.ManyToManyField(editable=False, to=settings.AUTH_USER_MODEL, verbose_name='users shown')),
            ],
            options={
                'verbose_name': 'tour',
                'verbose_name_plural': 'tours',
            },
        ),
        migrations.CreateModel(
            name='TourStep',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('step_id', models.CharField(max_length=50, verbose_name='step id')),
                ('title', models.CharField(max_length=100, verbose_name='title')),
                ('text', models.TextField(blank=True, verbose_name='text')),
                ('attach_to_selector', models.CharField(max_length=200, verbose_name='element selector')),
                ('attach_to_position', models.CharField(choices=[('TOP', 'Top'), ('BOTTOM', 'Bottom'), ('LEFT', 'left'), ('RIGHT', 'Right')], default='BOTTOM', max_length=10, verbose_name='attach to position')),
                ('tour', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='steps', to='tours.tour', verbose_name='tour')),
            ],
            options={
                'verbose_name': 'tour step',
                'verbose_name_plural': 'tour steps',
            },
        ),
    ]



from django.db import models
from django.utils.translation import gettext as _, gettext_lazy as _l
from django.contrib.auth import get_user_model


User = get_user_model()


class Tour(models.Model):
    name = models.CharField(
        _l('name'),
        max_length=100,
        unique=True,
    )
    description = models.CharField(
        max_length=200,
        blank=True,
    )
    url_names = models.CharField(
        _l('url names'),
        max_length=500,
        blank=True,
        help_text=_l('Comma separated list of url_name where you want it to be '
            'displayed. If not specified, it will be displayed on all pages.')
    )
    show_only_staff = models.BooleanField(
        _l('show only staff users'),
        default=False,
    )
    show_only_superuser = models.BooleanField(
        _l('show only staff users'),
        default=False,
    )
    start_date = models.DateField(
        _l('start date'),
        null=True,
        blank=True,
    )
    end_date = models.DateField(
        _l('end date'),
        null=True,
        blank=True,
    )
    timeout = models.PositiveIntegerField(
        default=1000,
    )
    use_modal_overlay = models.BooleanField(
        default=True,
    )
    is_active = models.BooleanField(
        default=True,
    )
    users_shown = models.ManyToManyField(
        User,
        verbose_name=_l('users shown'),
        editable=False,
    )

    class Meta:
        verbose_name = _l('tour')
        verbose_name_plural = _l('tours')

    def __str__(self):
        return self.name


class TourStep(models.Model):
    POSITION_TOP = 'TOP'
    POSITION_BOTTOM = 'BOTTOM'
    POSITION_LEFT = 'LEFT'
    POSITION_RIGHT = 'RIGHT'
    POSITION_CHOICES = (
        (POSITION_TOP, _l('Top')),
        (POSITION_BOTTOM, _l('Bottom')),
        (POSITION_LEFT, _l('left')),
        (POSITION_RIGHT, _l('Right')),
    )

    tour = models.ForeignKey(
        Tour,
        on_delete=models.CASCADE,
        related_name='steps',
        verbose_name=_l('tour'),
    )
    step_id = models.CharField(
        _l('step id'),
        max_length=50,
    )
    title = models.CharField(
        _l('title'),
        max_length=100,
    )
    text = models.TextField(
        _l('text'),
        blank=True,
    )
    attach_to_selector = models.CharField(
        _l('element selector'),
        max_length=200,
    )
    attach_to_position = models.CharField(
        _l('attach to position'),
        max_length=10,
        default=POSITION_BOTTOM,
        choices=POSITION_CHOICES,
    )

    class Meta:
        verbose_name = _l('tour step')
        verbose_name_plural = _l('tour steps')

    def __str__(self):
        return '%s %s' % (self._meta.verbose_name.capitalize(), self.step_id)



from django import forms
from django.apps import apps

from .models import Tour, TourStep


class TourForm(forms.ModelForm):
    class Meta:
        model = Tour
        fields = (
            'name',
            'description',
            'url_names',
            'show_only_staff',
            'show_only_superuser',
            'start_date',
            'end_date',
            'is_active',
        )
        widgets = {
            'description': forms.Textarea(),
            'url_names': forms.Textarea(),
        }


class TourStepForm(forms.ModelForm):
    class Meta:
        model = TourStep
        fields = (
            'tour',
            'step_id',
            'title',
            'text',
            'attach_to_selector',
            'attach_to_position',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if apps.is_installed('tinymce'):
            from tinymce.widgets import TinyMCE
            self.fields['text'].widget = TinyMCE()
